@@ -1,32 +1,32 @@
 # encoding=utf-8
 import contextlib
+import gzip
 import hashlib
 from http import cookiejar
 import logging
 import os
 import socket
 import sys
+from tempfile import TemporaryDirectory
 import tempfile
+import unittest
 
+from tornado.testing import AsyncHTTPSTestCase
 import tornado.testing
+from trollius import From, Return
+import trollius
 
-import wpull.backport.gzip
-from wpull.backport.testing import unittest
 from wpull.builder import Builder
 from wpull.dns import Resolver
 from wpull.errors import ExitStatus
 from wpull.options import AppArgumentParser
+from wpull.testing.async import AsyncTestCase
+import wpull.testing.async
 from wpull.testing.badapp import BadAppTestCase
 from wpull.testing.goodapp import GoodAppTestCase
 from wpull.url import URLInfo
 
 
-try:
-    from tempfile import TemporaryDirectory
-except ImportError:
-    from wpull.backport.tempfile import TemporaryDirectory
-
-
 DEFAULT_TIMEOUT = 30
 
 
@@ -38,10 +38,10 @@ class MockDNSResolver(Resolver):
         Resolver.__init__(self, *args, **kwargs)
         self.hosts_touched = set()
 
-    @tornado.gen.coroutine
+    @trollius.coroutine
     def resolve(self, host, port):
         self.hosts_touched.add(host)
-        raise tornado.gen.Return((socket.AF_INET, ('127.0.0.1', port)))
+        raise Return((socket.AF_INET, ('127.0.0.1', port)))
 
 
 @contextlib.contextmanager
@@ -60,7 +60,6 @@ class TestApp(GoodAppTestCase):
         self._original_cookiejar_debug = cookiejar.debug
         cookiejar.debug = True
         super().setUp()
-        tornado.ioloop.IOLoop.current().set_blocking_log_threshold(0.5)
         self.original_loggers = list(logging.getLogger().handlers)
 
     def tearDown(self):
@@ -71,12 +70,12 @@ class TestApp(GoodAppTestCase):
             if handler not in self.original_loggers:
                 logging.getLogger().removeHandler(handler)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_no_args(self):
         arg_parser = AppArgumentParser(real_exit=False)
         self.assertRaises(ValueError, arg_parser.parse_args, [])
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_one_page(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([self.get_url('/')])
@@ -84,10 +83,10 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
             self.assertTrue(os.path.exists('index.html'))
 
-            response = yield self.http_client.fetch(self.get_url('/'))
+            response = yield From(tornado_future_adapter(self.http_client.fetch(self.get_url('/'))))
 
             with open('index.html', 'rb') as in_file:
                 self.assertEqual(response.body, in_file.read())
@@ -101,7 +100,7 @@ class TestApp(GoodAppTestCase):
         self.assertEqual('hi', cookies[0].name)
         self.assertEqual('hello', cookies[0].value)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_big_payload(self):
         hash_obj = hashlib.sha1(b'foxfoxfox')
         payload_list = []
@@ -121,7 +120,7 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
             self.assertTrue(os.path.exists('big_payload'))
 
             with open('big_payload', 'rb') as in_file:
@@ -130,7 +129,7 @@ class TestApp(GoodAppTestCase):
         self.assertEqual(0, exit_code)
         self.assertEqual(1, builder.factory['Statistics'].files)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_many_page_with_some_fail(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -144,13 +143,13 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(ExitStatus.server_error, exit_code)
         self.assertGreater(builder.factory['Statistics'].files, 1)
         self.assertGreater(builder.factory['Statistics'].duration, 3)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_args(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -190,7 +189,7 @@ class TestApp(GoodAppTestCase):
         with cd_tempdir():
             builder = Builder(args)
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
             print(list(os.walk('.')))
             self.assertTrue(os.path.exists(
@@ -204,7 +203,7 @@ class TestApp(GoodAppTestCase):
         self.assertEqual(0, exit_code)
         self.assertEqual(builder.factory['Statistics'].files, 2)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_input_file_arg(self):
         arg_parser = AppArgumentParser(real_exit=False)
         with tempfile.NamedTemporaryFile() as in_file:
@@ -219,12 +218,12 @@ class TestApp(GoodAppTestCase):
             with cd_tempdir():
                 builder = Builder(args)
                 app = builder.build()
-                exit_code = yield app.run()
+                exit_code = yield From(app.run())
 
         self.assertEqual(0, exit_code)
         self.assertEqual(builder.factory['Statistics'].files, 2)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_input_html_file_arg(self):
         arg_parser = AppArgumentParser(real_exit=False)
         with tempfile.NamedTemporaryFile() as in_file:
@@ -242,12 +241,12 @@ class TestApp(GoodAppTestCase):
             with cd_tempdir():
                 builder = Builder(args)
                 app = builder.build()
-                exit_code = yield app.run()
+                exit_code = yield From(app.run())
 
         self.assertEqual(0, exit_code)
         self.assertEqual(builder.factory['Statistics'].files, 2)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_args_warc_size(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -262,7 +261,7 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
             self.assertTrue(os.path.exists('test-00000.warc.gz'))
             self.assertTrue(os.path.exists('test-meta.warc.gz'))
@@ -271,7 +270,7 @@ class TestApp(GoodAppTestCase):
         self.assertEqual(0, exit_code)
         self.assertGreaterEqual(builder.factory['Statistics'].files, 1)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_args_warc(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -288,18 +287,18 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
             self.assertTrue(os.path.exists('test.warc.gz'))
 
-            with wpull.backport.gzip.GzipFile('test.warc.gz') as in_file:
+            with gzip.GzipFile('test.warc.gz') as in_file:
                 data = in_file.read()
                 self.assertIn(b'FINISHED', data)
 
         self.assertEqual(0, exit_code)
         self.assertGreaterEqual(builder.factory['Statistics'].files, 1)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_args_warc_with_cdx(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -314,11 +313,11 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
         self.assertEqual(0, exit_code)
         self.assertGreaterEqual(builder.factory['Statistics'].files, 1)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_args_warc_dedup(self):
         arg_parser = AppArgumentParser()
 
@@ -343,7 +342,7 @@ class TestApp(GoodAppTestCase):
 
             builder = Builder(args)
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
             with open('test.warc', 'rb') as in_file:
                 data = in_file.read()
@@ -355,7 +354,7 @@ class TestApp(GoodAppTestCase):
         self.assertEqual(0, exit_code)
         self.assertGreaterEqual(builder.factory['Statistics'].files, 1)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_args_post_data(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -365,10 +364,10 @@ class TestApp(GoodAppTestCase):
         with cd_tempdir():
             builder = Builder(args)
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
         self.assertEqual(0, exit_code)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_sanity(self):
         arg_items = [
             ('--verbose', '--quiet'),
@@ -402,7 +401,7 @@ class TestApp(GoodAppTestCase):
             else:
                 self.assertTrue(False)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_python_script(self):
         arg_parser = AppArgumentParser()
         filename = os.path.join(os.path.dirname(__file__),
@@ -418,7 +417,7 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(42, exit_code)
 
@@ -428,7 +427,7 @@ class TestApp(GoodAppTestCase):
         stats = builder.factory['Statistics']
         self.assertGreater(1.0, stats.duration)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_python_script_api_2(self):
         arg_parser = AppArgumentParser()
         filename = os.path.join(os.path.dirname(__file__),
@@ -444,7 +443,7 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(42, exit_code)
 
@@ -454,7 +453,7 @@ class TestApp(GoodAppTestCase):
         stats = builder.factory['Statistics']
         self.assertGreater(1.0, stats.duration)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_python_script_stop(self):
         arg_parser = AppArgumentParser()
         filename = os.path.join(os.path.dirname(__file__),
@@ -466,13 +465,13 @@ class TestApp(GoodAppTestCase):
         with cd_tempdir():
             builder = Builder(args)
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(1, exit_code)
 
     @unittest.skipIf(sys.version_info[0:2] == (3, 2),
                      'lua module not working in this python version')
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_lua_script(self):
         arg_parser = AppArgumentParser()
         filename = os.path.join(os.path.dirname(__file__),
@@ -488,7 +487,7 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(42, exit_code)
 
@@ -500,7 +499,7 @@ class TestApp(GoodAppTestCase):
 
     @unittest.skipIf(sys.version_info[0:2] == (3, 2),
                      'lua module not working in this python version')
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_lua_script_api_2(self):
         arg_parser = AppArgumentParser()
         filename = os.path.join(os.path.dirname(__file__),
@@ -516,7 +515,7 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(42, exit_code)
 
@@ -526,17 +525,28 @@ class TestApp(GoodAppTestCase):
         stats = builder.factory['Statistics']
         self.assertGreater(1.0, stats.duration)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_iri_handling(self):
         arg_parser = AppArgumentParser()
-        args = arg_parser.parse_args([self.get_url('/static/mojibake.html')])
+        args = arg_parser.parse_args([
+            self.get_url('/static/mojibake.html'),
+            '-r',
+            '--database', 'temp-unittest.db'
+        ])
         with cd_tempdir():
             builder = Builder(args)
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
+
+            urls = list(iter(builder.factory['URLTable']))
+            self.assertIn(
+                self.get_url('/%E6%96%87%E5%AD%97%E5%8C%96%E3%81%91'),
+                urls
+            )
+
         self.assertEqual(0, exit_code)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_cookie(self):
         arg_parser = AppArgumentParser()
 
@@ -557,7 +567,7 @@ class TestApp(GoodAppTestCase):
 
             with cd_tempdir():
                 app = builder.build()
-                exit_code = yield app.run()
+                exit_code = yield From(app.run())
 
                 self.assertEqual(0, exit_code)
                 self.assertEqual(1, builder.factory['Statistics'].files)
@@ -573,7 +583,7 @@ class TestApp(GoodAppTestCase):
 
                 self.assertIn(b'test\tyes', cookie_data)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_redirect_diff_host(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -586,7 +596,7 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(0, exit_code)
         self.assertEqual(1, builder.factory['Statistics'].files)
@@ -594,7 +604,7 @@ class TestApp(GoodAppTestCase):
         resolver = builder.factory['Resolver']
         self.assertIn('somewhereelse.invalid', resolver.hosts_touched)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_redirect_diff_host_recursive(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -608,14 +618,14 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
         self.assertEqual(0, exit_code)
         self.assertEqual(1, builder.factory['Statistics'].files)
 
         resolver = builder.factory['Resolver']
         self.assertIn('somewhereelse.invalid', resolver.hosts_touched)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_redirect_span_hosts_allow_linked(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -631,14 +641,14 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
         self.assertEqual(0, exit_code)
         self.assertEqual(2, builder.factory['Statistics'].files)
 
         resolver = builder.factory['Resolver']
         self.assertIn('linked.test', resolver.hosts_touched)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_redirect_span_hosts_page_requisites(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -654,14 +664,14 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
         self.assertEqual(0, exit_code)
         self.assertEqual(2, builder.factory['Statistics'].files)
 
         resolver = builder.factory['Resolver']
         self.assertIn('pagereq.test', resolver.hosts_touched)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_strong_redirect(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -676,14 +686,14 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
         self.assertEqual(0, exit_code)
         self.assertEqual(0, builder.factory['Statistics'].files)
 
         resolver = builder.factory['Resolver']
         self.assertNotIn('somewhereelse.invalid', resolver.hosts_touched)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_immediate_robots_fail(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -699,12 +709,12 @@ class TestApp(GoodAppTestCase):
                 URLInfo.parse(self.get_url('/')),
                 'User-Agent: *\nDisallow: *\n'
             )
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(0, exit_code)
         self.assertEqual(0, builder.factory['Statistics'].files)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_immediate_robots_forbidden(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -715,12 +725,12 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(0, exit_code)
         self.assertEqual(0, builder.factory['Statistics'].files)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_quota(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -733,12 +743,12 @@ class TestApp(GoodAppTestCase):
             builder = Builder(args)
 
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(0, exit_code)
         self.assertEqual(1, builder.factory['Statistics'].files)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_content_on_error(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -750,7 +760,7 @@ class TestApp(GoodAppTestCase):
             builder = Builder(args)
 
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
             print(list(os.walk('.')))
             self.assertTrue(os.path.exists('always_error'))
@@ -758,7 +768,7 @@ class TestApp(GoodAppTestCase):
         self.assertEqual(0, exit_code)
         self.assertEqual(1, builder.factory['Statistics'].files)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_phantomjs(self):
         arg_parser = AppArgumentParser()
         script_filename = os.path.join(os.path.dirname(__file__),
@@ -780,7 +790,7 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
             self.assertTrue(os.path.exists('test.warc'))
             self.assertTrue(
@@ -812,7 +822,7 @@ class TestApp(GoodAppTestCase):
         self.assertEqual(0, exit_code)
         self.assertGreaterEqual(builder.factory['Statistics'].files, 1)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_app_phantomjs_scroll(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -827,7 +837,7 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
             with open('DEUUEAUGH.html.snapshot.html', 'rb') as in_file:
                 data = in_file.read()
@@ -835,7 +845,7 @@ class TestApp(GoodAppTestCase):
 
         self.assertEqual(0, exit_code)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_sitemaps(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -849,7 +859,7 @@ class TestApp(GoodAppTestCase):
             builder = Builder(args)
 
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
             print(list(os.walk('.')))
             self.assertTrue(os.path.exists(
@@ -860,7 +870,7 @@ class TestApp(GoodAppTestCase):
         self.assertEqual(0, exit_code)
         self.assertGreaterEqual(4, builder.factory['Statistics'].files)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_local_encoding(self):
         arg_parser = AppArgumentParser()
 
@@ -882,12 +892,12 @@ class TestApp(GoodAppTestCase):
 
             with cd_tempdir():
                 app = builder.build()
-                exit_code = yield app.run()
+                exit_code = yield From(app.run())
 
         self.assertEqual(0, exit_code)
         self.assertEqual(2, builder.factory['Statistics'].files)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_no_iri(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -899,12 +909,12 @@ class TestApp(GoodAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(0, exit_code)
         self.assertEqual(1, builder.factory['Statistics'].files)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_output_document(self):
         arg_parser = AppArgumentParser()
 
@@ -916,7 +926,7 @@ class TestApp(GoodAppTestCase):
 
             builder = Builder(args)
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
             self.assertTrue(os.path.exists('blah.dat'))
 
@@ -928,13 +938,24 @@ class SimpleHandler(tornado.web.RequestHandler):
         self.write(b'OK')
 
 
-class TestAppHTTPS(tornado.testing.AsyncHTTPSTestCase):
+class TestAppHTTPS(AsyncTestCase, AsyncHTTPSTestCase):
+    def get_new_ioloop(self):
+        tornado.ioloop.IOLoop.configure(
+            'wpull.testing.async.TornadoAsyncIOLoop',
+            event_loop=self.event_loop)
+        ioloop = tornado.ioloop.IOLoop()
+        return ioloop
+
+    def setUp(self):
+        AsyncTestCase.setUp(self)
+        AsyncHTTPSTestCase.setUp(self)
+
     def get_app(self):
         return tornado.web.Application([
             (r'/', SimpleHandler)
         ])
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_https_only(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -948,18 +969,14 @@ class TestAppHTTPS(tornado.testing.AsyncHTTPSTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(0, exit_code)
         self.assertEqual(1, builder.factory['Statistics'].files)
 
 
 class TestAppBad(BadAppTestCase):
-    def setUp(self):
-        super().setUp()
-        tornado.ioloop.IOLoop.current().set_blocking_log_threshold(0.5)
-
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_bad_cookie(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -969,7 +986,7 @@ class TestAppBad(BadAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
         self.assertEqual(0, exit_code)
         self.assertEqual(1, builder.factory['Statistics'].files)
 
@@ -977,7 +994,7 @@ class TestAppBad(BadAppTestCase):
         _logger.debug('{0}'.format(cookies))
         self.assertEqual(2, len(cookies))
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_long_cookie(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -987,7 +1004,7 @@ class TestAppBad(BadAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
         self.assertEqual(0, exit_code)
         self.assertEqual(1, builder.factory['Statistics'].files)
 
@@ -995,7 +1012,7 @@ class TestAppBad(BadAppTestCase):
         _logger.debug('{0}'.format(cookies))
         self.assertEqual(0, len(cookies))
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_non_http_redirect(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -1007,12 +1024,12 @@ class TestAppBad(BadAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(0, exit_code)
         self.assertEqual(0, builder.factory['Statistics'].files)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_bad_redirect(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -1025,12 +1042,12 @@ class TestAppBad(BadAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(7, exit_code)
         self.assertEqual(0, builder.factory['Statistics'].files)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_ignore_length(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -1042,12 +1059,12 @@ class TestAppBad(BadAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(0, exit_code)
         self.assertEqual(1, builder.factory['Statistics'].files)
 
-    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    @wpull.testing.async.async_test(timeout=DEFAULT_TIMEOUT)
     def test_bad_utf8(self):
         arg_parser = AppArgumentParser()
         args = arg_parser.parse_args([
@@ -1061,7 +1078,18 @@ class TestAppBad(BadAppTestCase):
 
         with cd_tempdir():
             app = builder.build()
-            exit_code = yield app.run()
+            exit_code = yield From(app.run())
 
         self.assertEqual(0, exit_code)
         self.assertEqual(4, builder.factory['Statistics'].files)
+
+
+@trollius.coroutine
+def tornado_future_adapter(future):
+    event = trollius.Event()
+
+    future.add_done_callback(lambda dummy: event.set())
+
+    yield From(event.wait())
+
+    raise Return(future.result())
