@@ -222,7 +222,7 @@ class Callbacks(LegacyCallbacks):
         Args:
             host (str): The hostname.
 
-        This callback is to override the DNS response.
+        This callback is to override the DNS lookup.
 
         It is useful when the server is no longer available to the public.
         Typically, large infrastructures will change the DNS settings to
@@ -232,7 +232,7 @@ class Callbacks(LegacyCallbacks):
 
         Returns:
             str, None: ``None`` to use the original behavior or a string
-            containing an IP address.
+            containing an IP address or an alternate hostname.
         '''
         return None
 
@@ -260,6 +260,28 @@ class Callbacks(LegacyCallbacks):
         '''
         return verdict
 
+    @staticmethod
+    def queued_url(url_info):
+        '''Callback fired after an URL was put into the queue.
+
+        Args:
+            url_info (dict): A mapping containing the same information in
+                :class:`.url.URLInfo`.
+        '''
+        pass
+
+    @staticmethod
+    def dequeued_url(url_info, record_info):
+        '''Callback fired after an URL was retrieved from the queue.
+
+        Args:
+            url_info (dict): A mapping containing the same information in
+                :class:`.url.URLInfo`.
+            record_info (dict): A mapping containing the same information in
+                :class:`.item.URLRecord`.
+        '''
+        pass
+
     @staticmethod
     def handle_response(url_info, record_info, http_info):
         '''Return an action to handle the response.
@@ -457,6 +479,9 @@ class HookEnvironment(object):
 
         self.factory['Resolver'].connect_hook('resolve_dns', self._resolve_dns)
         self.factory['Engine'].connect_hook('engine_run', self._engine_run)
+        self.factory['Engine'].connect_hook(
+            'dequeued_url',
+            self._dequeued_url)
         self.factory['Application'].connect_hook(
             'exit_status', self._exit_status
         )
@@ -464,6 +489,9 @@ class HookEnvironment(object):
             'finishing_statistics', self._finishing_statistics
         )
         self.factory['WebProcessor'].connect_hook('wait_time', self._wait_time)
+        self.factory['WebProcessor'].connect_hook(
+            'queued_url',
+            self._queued_url)
         self.factory['WebProcessor'].connect_hook(
             'should_fetch',
             self._should_fetch)
@@ -507,6 +535,19 @@ class HookEnvironment(object):
             seconds = to_lua_type(seconds)
         return self.callbacks.wait_time(seconds)
 
+    def _queued_url(self, url_info):
+        url_info_dict = self.to_script_native_type(url_info.to_dict())
+
+        self.callbacks.queued_url(url_info_dict)
+
+    def _dequeued_url(self, url_info, url_record):
+        url_info_dict = self.to_script_native_type(url_info.to_dict())
+
+        record_info_dict = url_record.to_dict()
+        record_info_dict = self.to_script_native_type(record_info_dict)
+
+        self.callbacks.dequeued_url(url_info_dict, record_info_dict)
+
     def _should_fetch(self, url_info, url_record, verdict, reason_slug,
                       test_info):
         url_info_dict = self.to_script_native_type(url_info.to_dict())
@@ -585,7 +626,7 @@ class HookEnvironment(object):
         to_native = self.to_script_native_type
         url_info_dict = to_native(request.url_info.to_dict())
         document_info_dict = to_native(response.body.to_dict())
-        filename = to_native(response.body.content_file.name)
+        filename = to_native(response.body.name)
 
         new_url_dicts = self.callbacks.get_urls(
             filename, url_info_dict, document_info_dict)
@@ -622,8 +663,8 @@ class HookEnvironment(object):
         assert url
 
         # FIXME: resolve circular imports
-        from wpull.processor import WebProcessorSession
-        url_info = WebProcessorSession.parse_url(url, 'utf-8')
+        from wpull.processor.web import WebProcessorSession
+        url_info = WebProcessorSession.parse_url(url)
 
         if not url_info:
             return
@@ -634,6 +675,9 @@ class HookEnvironment(object):
             url_item.url_table.remove([url])
 
         if inline:
-            url_item.add_inline_url_infos([url_info], **kwargs)
+            added_url_infos = url_item.add_inline_url_infos([url_info], **kwargs)
         else:
-            url_item.add_linked_url_infos([url_info], **kwargs)
+            added_url_infos = url_item.add_linked_url_infos([url_info], **kwargs)
+
+        for url_info in added_url_infos:
+            self._queued_url(url_info)
